'DRAW93 instructions
'
'written by: Milo Sedlacek (a.k.a. gradius)
'            e-mail: sedlacek@execulink.com
'            IRC: irc.sprynet.com #quickbasic
'
'
'  This document is divided into several sections.
'
'1.1  Introduction
'1.2  Screen layout
'2.1  Feature list
'3.1  File formats
'4.1  Additional notes
'5.1  Credits
'
'
'
'   1.1  Introduction   
'
'
'
'  Draw93 is an image editor written for use with Qbasic 1.1 or QuickBasic.
'It works exclusively in the SCREEN 13 video mode (320x200 256 colours).
'Features include variable size frames (1x1 to 64x64), custom palettes and
'multiple frames in a single file, when .imx mode is enabled.
'
'
'
'   1.2  Screen Layout   
'
'
'
'This section describes draw93's screen areas.
'                             
'                                       draw colour
'                                       |
'                                       | erase colour
'                                       | |
'     ||Ŀ
'                                    | |  256  
'                                    | colour
'                                      | matrix
'                                    Ĵ
'                                     r ======
'                                     g ======--- displays rgb
'                                     b ======    of draw colour
'            Expanded Image          Ĵ
'                                     x     auto-----displayed when
'                                     y             auto draw is on
'                                     xsize     
'                                     ysize     
'                                     frame     
'                                     filename  
'         palette   
'                                            ---------message area
'     Ĵ
'                            page 1/1  Ĵ
'         Help area                          --------thumb print image
'                                               
'     
'
'
'
'   2.1  Feature list   
'
'
'
'  The following is a list of keys which initiate functions.
'
'ESC ESC        - exits program
'space bar or b - draws a pixel with the draw colour
'+-             - increase / decrease draw colour by one
'shift +-       - increase / decrease draw colour by 16
'x              - swap erase and draw colour
'shift x        - make erase colour equal draw colour
'/              - make draw colour equal colour under cursor
'?              - make erase colour equal colour under cursor
'
'f              - change filename
'
'                 When you press "f" a box will appear with which you will
'                 choose whether to edit the image or the palette filename.
'                 Press ESC to return to drawing or ENTER to edit the
'                 filename.  When editing the filename, press ENTER again
'                 when done.
'
'z              - change image size
'
'                 With this function you can change the dimensions of the
'                 image you are working with.  Unless you specify a size
'                 smaller than the one you're currently using, image data
'                 will not be lost.
'
'e              - draws a pixel with the erase colour
'~              - draws a pixel with colour 0
'c              - copies the current frame into a temporary buffer
'i              - transfers the frame from the temporary buffer to the
'                 current frame.  The previous image will be overwritten.
'
'left shift i   - transfers the temporary buffer over top of the current frame
'                 with transparency
'
'right shift i  - transfers the temporary buffer "underneath" the current frame
'r              - flips image horizontally
'left shift r   - flips image vertically
'right shift r  - rotates image 90 degrees clockwise
'd              - adjust frame xy location
'
'                 You can use this to shift the data in the current frame
'                 around.  Press ENTER to effect changes, or ESC to restore
'                 the image's original location
'
'n              - copy pixel from temp to cursor location.  This is useful
'                 when you're drawing animations.  Copy the current frame.
'                 To replace a background pixel if you mess up, press "n"
'
'w              - this function is almost exactly the same as the above, with
'                 an additional twist.  If you copy, say a 2x2 frame into the
'                 temp buffer and go to location 10,10 in another image and
'                 press "w", tiling will occur, as opposed to the "n" function
'                 where colour 0 would be used.  I apoligize if some of these
'                 funcitons do not make sense to you at this time.
'
'a              - this toggles "auto" draw.  When auto draw is on, a pixel
'                 be drawn with the draw colour at the cursor location every
'                 time you move.
'
'home           - make cursor jump to left edge
'end            - make cursor jump to right edge
'shift home     - make cursor jump to top edge
'shift end      - make cursor jump to bottom edge
'ctrl home      - make cursor jump to top left corner
'ctrl end       - make cursor jump to bottom right corner
'shift ctrl home- make cursor jump to top right corner
'shift ctrl end - make cursor jump to bottom left corner
'
'  ctrl home    shift home       shift ctrl home
'        \          |          /
'          
'                           
'                           
'   home --                 -- end
'                           
'                           
'          
'        /          |          \
'shift ctrl end    shift end     ctrl end
'
'
' Ok coffe break
'
' Alright
'
'l              - load frames or palette.  A box will pop up.  You can select
'                 whether you want to load bitmap data or the palette.
'
's              - save frames or palette.  See above.
'
'7 (keypad)     - decreases current red value in palette
'9 (keypad)     - increases current red value in palette
'4 (keypad)     - decreases current green value in palette
'6 (keypad)     - increases current green value in palette
'1 (keypad)     - decreases current blue value in palette
'3 (keypad)     - increases current blue value in palette
'8 (keypad)     - brighten rgb colour in palette
'2 (keypad)     - darken rpg colour in palette  *caution this function does
'                                                not work too well
'5 (keypad)     - set colour spread.  Pick a colour with +/-.  Press ENTER.
'                 pick another colour with +/-.  Press ENTER again.
'
'shift f        - flood fill.  An arbitary pixel area will be filled until
'                 a border of an un-equal value is found.  This is unlike
'                 the way qbasic's PAINT command works.
'
'p              - qbasic PAINT.  Draw colour = fill colour, erase colour =
'                 border colour.  This is great for wiping out entire areas
'                 of pixels.
'
'o              - outline the image with draw colour
'
'shift c        - change colours in frame.  This operation is performed on all
'                 the pixels in the frame.  Erase colour = colour to change,
'                 Draw colour = colour to change it to
'
'shift n        - clear frame.  You will be prompted to confirm this command.
'
'\              - remove border around sprite.  This will reduce the sprite's
'                 dimensions so that no border of blank pixels exists.
'
'insert         - inserts a "collumn" of pixels into the image at cursor's x
'                 location
'
'delete         - deletes a "collumn" of pixels from the image at cursor's x
'                 location
'
'shift insert   - insert a "row" of pixels into the image at cursor's y
'                 location
'
'shift delete   - deletes a "row" of pixels from the image at cursor's y
'                 location
'
']              - adds 1 to all colours in the image, excluding 0
'[              - subtracts 1 from all the colours in the image, except 0
'}              - adds 16 to all colour in the image, excluding 0
'{              - subtracts 16 from all the colours in the image, except 0
'm              - generate a "mask" from the frame.  All zero pixels will be
'                 made 255, all nonzero pixels will be made 0
'
'Special moves:
'
'- go up to an edge, hold shift and press in the direction of the edge.  This
'  will expand the frame accordingly
'
'- go up to an edge, hold shift and press away from the direction of the edge.
'  this will shrink the image accordingly.
'
'- to generate a "silhouette" image, select a colour, go into the interior of
'  an image and press "p".  This will make the whole image one colour.
'
'The following function only work when the filename ends with ".imx"
'
'F5             - insert a blank frame after current frame
'shift F5       - insert a blank frame before current frame
'F6             - delete current frame
'page up        - goto next frame
'page down      - go to previous frame
'shift page up  - go 5 frames forward
'shift page down- go 5 frames backward
'ctrl page up   - goto first frame
'ctrl page down - goto last frame
'
'
'
'   3.1  File formats   
'
'
'
'*.IMG file format:
'------------------
'
'  This is just a BSAVEd qbasic image.  Here are the bytes of the file.
'
'1-7     used by the BSAVE command
'8-9          image x size * 8
'10-11        image y size
'12-... ...    image data
'
'When you actually BLOAD this, the first 7 bytes in the file are dropped, so
'this is what the data structure looks like after it's loaded into memory
'
'0-1      image x size * 8
'2-3      image y size
'4-... ... image data
'
'Now you may be asking two questions:
'1.)  How do I load an *.IMG in my program, and
'2.)  Why is the x size multiplied by 8?
'
'to load an *.IMG:
'
'DIM SHARED image%(size%)
'DEF SEG = VARSEG(image%(0))
'BLOAD filename$, VARPTR(image%(0))
'DEF SEG
'
'    This will load the *.IMG into memory.  Pay careful attention to how much
'memory is allocated to store the image.  Look at the file size, with the DOS
'"dir" command, subtract 7 and divide the value by two.  Thus, a byte with a
'file size of 263 bytes would need 128 array elements.
'
'Now to display it using QBASIC, you need to do several things:
'
'1.) set the video mode with SCREEN 13
'2.) use PUT to draw the image on the screen
'
'The code follows:
'
'SCREEN 13
'PUT (x%, y%), image%
'
'Combined with the above example, the code to load and display an image is:
'
'DIM SHARED image%(size%)
'DEF SEG = VARSEG(image%(0))
'BLOAD filename$, VARPTR(image%(0))
'DEF SEG
'SCREEN 13
'PUT (x%, y%), image%
'
'After running this you will realize that the image's colours are off.  This
'is because the VGA's colour lookup table has not been set yet.  This brings
'us to....
'
'the *.PAL format
'
'Firstly, realize that there are many file formats ending with *.PAL.  This
'is not the 'official' *.PAL file specification.
'
'1-7    used by QBASIC's BSAVE
'8            colour 0 red
'9            colour 0 green
'10           colour 0 blue
'...
'773          colour 255 red
'774          colour 255 green
'775          colour 255 blue
'
'This brings us to another aspect of *.PAL files.
'
'some *.PAL files, such as the ones used in "Marconian Dreams" have a blank
'byte after each palette entry, so the entire file is 1031 bytes instead of
'775.
'
'In order to load a *.PAL file, the following code is required:
'
'SCREEN 13
'DIM SHARED pal%(384)
'DEF SEG = VARSEG(pal%(0))
'i& = VARPTR(pal%(0))
'BLOAD palfile$, i&
'FOR c% = 0 to 255
'  OUT &H3c8, c%
'  OUT &H3c9, PEEK(i&)
'  OUT &H3c9, PEEK(i& + 1)
'  OUT &H3c9, PEEK(i& + 2)
'  i& = i& + 3
'NEXT
'DEF SEG
'
'Combining the code of loading an image with a palette gives us the following:
'
'SCREEN 13
'
'DIM SHARED image%(size%)
'DIM SHARED pal%(384)
'
'DEF SEG = VARSEG(pal%(0))
'i& = VARPTR(pal%(0))
'BLOAD palfile$, i&
'FOR c% = 0 to 255
'  OUT &H3c8, c%
'  OUT &H3c9, PEEK(i&)
'  OUT &H3c9, PEEK(i& + 1)
'  OUT &H3c9, PEEK(i& + 2)
'  i& = i& + 3
'NEXT
'DEF SEG
'
'DEF SEG = VARSEG(image%(0))
'BLOAD filename$, VARPTR(image%(0))
'DEF SEG
'
'PUT (x%, y%), image%
'
'Notice as a program increases in size the code becomes more clearly written.
'
'The *.IMX file format
'
'This is a multi-size, multi-frame format that works very well with qbasic.
'With a single line of code, you can display any frame.
'
'Here is the file, once loaded into memory:
'
'0-1                   number of frames in *.IMX file
'2-15      unused
'16-17                 element number (offset) of frame 0
'18-19                 element number (offset) of frame 1
'20...
'?-?+1                 element number (offset) of frame n
'?-?+1                 frame 0 x size * 8
'?+2-?+3               frame 0 y size
'?+4-... ...            frame 0 pixel data
'...
'
'This may look confusing at first, but once studying it closely you will rea-
'lize that it is possible to be understood.
'
'To load an *.IMX into memory do the following:
'
'DIM SHARED imx%(size%)
'DEF SEG = VARSEG(imx%(0))
'BLOAD imxfile$, VARPTR(imx%(0))
'DEF SEG
'
'To display any frame of an imx, do the following:
'
'SCREEN 13
'PUT (x%, y%), imx%(imx%(8 + frame%))
'
'Note that the frames start at 0.  If in draw93 you see you're using frame
'10, in your program where you load the *.IMX, frame% = 9.
'
'Combining this code with the code from the section above, we have a *.PAL
'and *.IMX loader:
'
'SCREEN 13
'
'DIM SHARED imx%(imxsize%)
'DIM SHARED pal%(384)
'
'DEF SEG = VARSEG(pal%(0))
'i& = VARPTR(pal%(0))
'BLOAD palfile$, i&
'FOR c% = 0 to 255
'  OUT &H3c8, c%
'  OUT &H3c9, PEEK(i&)
'  OUT &H3c9, PEEK(i& + 1)
'  OUT &H3c9, PEEK(i& + 2)
'  i& = i& + 3
'NEXT
'DEF SEG
'
'DEF SEG = VARSEG(imx%(0))
'BLOAD imxfile$, VARPTR(imx%(0))
'DEF SEG
'
'PUT (x%, y%), imx%(imx%(8 + frame%))
'
'
'
'   4.1  Additional notes   
'
'
'
'Note that Draw93 has a memory limit set to 24K.  In order to use more than
'24 K, you'll need to run QuickBasic 4.5 with "qb /l" in the command line.
'Set the IMXWORDSIZE constant to 32766 to use the full 64K.  An *.IMX is
'never larger than 64K because it's BSAVEd.
'
'
'
'   4.1  Credits   
'
'
'
'ANGEL.IMG, supplied with DRAW93.ZIP was drawn by Lukas Sedlacek
'the default palette for DRAW93 was designed by Ryan Ross (a.k.a. GamerX)
'
'Thaks for reading this, I do appreciate it!
'
'Heya send me some mail:  sedlacek@execulink.com
'
':P~~~~

